@@ -4,8 +4,9 @@ use warnings;
 use base qw( CatalystX::CRUD::Controller );
 use Carp;
 use Data::Dump;
+use Class::C3;
 
-our $VERSION = '0.28';
+our $VERSION = '0.29';
 
 =head1 NAME
 
@@ -113,7 +114,8 @@ Returns a new C<form_class> object every time, initialized with C<form_fields>.
 sub form {
     my ( $self, $c ) = @_;
     my $form_class = $self->config->{form_class};
-    my $form = $form_class->new( { fields => $self->config->{form_fields} } );
+    my $arg        = { fields => $self->config->{form_fields} };
+    my $form       = $form_class->new($arg);
     return $form;
 }
 
@@ -133,8 +135,11 @@ sub end : Private {
     if ( defined $c->stash->{object} ) {
         $c->res->body( $self->serialize_object( $c, $c->stash->{object} ) );
     }
-    if ( @{ $c->error } ) {
-        $c->res->body( join( "\n", @{ $c->error } ) );
+    if ( $self->has_errors($c) ) {
+        my $err = join( "\n", @{ $c->error } );
+        $c->log->error($err) if $c->debug;
+        $c->res->body($err);
+        $c->res->status(500);
         $c->clear_errors;
     }
 }
